/* $Id: mainwin.c,v 1.19 2000/02/12 14:57:40 rsmith Exp $
 * This file is part of xplanets
 * Copyright (C) 2000  R.F. Smith
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * You can contact the author at the following addresses:
 *     e-mail: rsmith@xs4all.nl
 * snail-mail: R.F. Smith
 *             Dr. Hermansweg 36
 *             5624 HR Eindhoven
 *             The Netherlands
 *
 * $Log: mainwin.c,v $
 * Revision 1.19  2000/02/12 14:57:40  rsmith
 * Removed orbit drawing code: too slow.
 *
 * Revision 1.18  2000/02/01 21:22:58  rsmith
 * Added code to draw planetary orbits. (see orbits.c)
 *
 * Revision 1.17  2000/01/30 17:00:40  rsmith
 * Check for crashing into a planet.
 * Added crash dialog.
 * Adapted to r_screen from planet.*
 *
 * Revision 1.16  2000/01/29 19:29:00  rsmith
 * Changed the radius for detecting a planet from 25 to 2*planet_screen_size.
 *
 * Revision 1.15  2000/01/29 19:09:04  rsmith
 * Added planet dialog, activated by mouse click.
 * Added true planet resizing (visible at small window sizes).
 *
 * Revision 1.14  2000/01/28 20:01:50  rsmith
 * Removed #include of gtkdial.h
 *
 * Revision 1.13  2000/01/28 19:46:43  rsmith
 * Changed the control interface. It is now possible to specify
 * the accelleration direction with the mouse or a spin button.
 * The magnitude of the accelleration can be controlled by a scale.
 *
 * Revision 1.12  2000/01/27 21:05:48  rsmith
 * The velocity vector scaling factor is now #defined.
 * The ships' delta v vector is now green, and shows the length.
 * Replaced the dial and engine button by a pair of spin buttons that indicate
 * the direction and length of the delta v vector of a burn.
 * The delta v vector can also be specified by clicking in the picture.
 *
 * Revision 1.11  2000/01/23 11:14:03  rsmith
 * Adapted to the new view settings (The view now centers on the ship).
 * Added the "data" frame.
 *
 * Revision 1.10  2000/01/16 09:21:06  rsmith
 * The relative velocity vectors now also indicate the magnitude of the velocity.
 *
 * Revision 1.9  2000/01/14 20:00:11  rsmith
 * - Added display of relative speeds.
 * - Release 0.3.4
 *
 * Revision 1.8  2000/01/09 18:25:10  rsmith
 * - Fixed core dump bug. (6th argument to gdk_colors_alloc should be an array
 *   instead of a single value).
 * - Release 0.3.3
 *
 * Revision 1.7  2000/01/09 12:18:39  rsmith
 * - Added disabling of zoom_in or zoom_out button when zooming is not
 *   possible.
 * - Frozen for release 0.3.2
 *
 * Revision 1.6  2000/01/09 12:01:14  rsmith
 * - Disabled dial and engine_btn when simulation is not running.
 * - Frozen for release 0.3.1
 *
 * Revision 1.5  2000/01/09 10:45:50  rsmith
 * Frozen for the 0.3.0 release
 *
 * Revision 1.4  2000/01/09 10:34:20  rsmith
 * - Non zero return value bug fixed: removed gtk_exit as clicked signal
 *   handler from the quit button.
 *
 * Revision 1.3  2000/01/07 13:35:11  rsmith
 * First working simulation!
 *
 * Revision 1.2  2000/01/06 18:04:06  rsmith
 * First working UI!
 *
 * Revision 1.1  2000/01/05 21:40:43  rsmith
 * Initial revision
 *
 */

#include <string.h>
#include <stdio.h>
#include <math.h>
#include <gtk/gtk.h>
#include "planets.h"
#include "about.h"

#include <sys/time.h>
#include <unistd.h>



/* initial size request for drawing area */
#define DA_SIZE 400
/* relative velocity scaling factor */
#define V_SCALE 10
/* velocity vector length */
#define V_LEN 20

/* timeout interval [milliseconds] */
#define TMOUT 100
#define A 9.81          /* ship's accelleration (1G) [m/s^2] */

/*************** IMPORTS **************/
void show_planet_dialog (int i); /* from planetdlg.c */
void show_crash_dialog (int i);  /* from crashdlg.c */
/*  ci_t *orbit_get_visible_section (double r); */ /* from orbits.c */

/*************** EXPORTS **************/
GtkWidget *new_btn = NULL;
GtkWidget *run_toggle = NULL;

/************ STATIC DATA *************/
/* window size is 10^zoomlevel meter. */
enum {
  MIN_ZOOM = 6,
  MAX_ZOOM = 14
};
static double zoomlevel = 12;
/* simulation status */
enum {
  SIM_STOP = 0, /* simulation not running */
  SIM_RUN,      /* simulation running */
  SIM_PAUZE     /* simulation pauzed */
};
static gint sim_state = SIM_STOP;
/* indicates if the ship is in the center of view */
static gboolean center_ship = TRUE;
/* indicates if hte planets' orbits should be drawn*/
/*  static gboolean draw_orbits = TRUE; */
/* simulation time speedup factor (sim. time / real time) */
static double time_acc_fact = 1;
/* index of the planet closest to the ship. */
static gint min_dist_index = 0;

/* graphic context for drawing into pixmap */
static GdkGC *pmgc = NULL;
/* backing pixmap for drawing area */
static GdkPixmap *pixmap = NULL;
/* widgets */
static GtkWidget *zoom_in_btn = NULL;
static GtkWidget *zoom_out_btn = NULL;
static GtkWidget *burn_toggle = NULL;
static GtkWidget *draw_area = NULL;
static GtkWidget *scale_label = NULL;
static GtkWidget *planet_name_label = NULL;
static GtkWidget *planet_dist_label = NULL;
static GtkWidget *planet_speed_label = NULL;
/* direction adjustment */
static GtkObject *acc_adjustment = NULL;         /* adjustment = G */
static GtkObject *angle_adjustment = NULL;      /* adjustment = degrees */

/* color of the ship marker */
static GdkColor ship_color = {0, 65535, 65535, 65535};
/* color for the closest planet indicator */
static GdkColor prox_color = {0, 65535, 0, 0};
/* color for the ships' accelleration vector. */
static GdkColor line_color = {0, 0, 65535, 0};
/* color for the planets' orbits. */
static GdkColor orbit_color = {0, 0xbefb, 0xbefb, 0xbefb};
/* colors of the planets */
static GdkColor color[NUM_PLANETS] = {
  {0, 65535, 65535,     0},     /* sun      255, 255,  0 - yellow */
  {0, 24929, 24929, 24929},     /* mercury  97,  97,  97 - gray38 */
  {0, 61166, 50629, 37265},     /* venus    238, 197, 145 - burlywood2 */
  {0, 0,     0,     61166},     /* earth    0,   0,   238 - blue2 */
  {0, 52685, 33153, 25186},     /* mars     205, 129,  98 - LightSalmon3 */
  {0, 52685, 26214,  7453},     /* jupiter  205, 102,  29 - chocolate3 */
  {0, 61166, 46260,  8738},     /* saturn   238, 180,  34 - goldenrod2 */
  {0, 45746, 57311, 61166},     /* uranus   178, 223, 238 - LightBlue2  */
  {0, 25700, 38293, 60909},     /* neptune  100, 149, 237 - cornflower blue */
  {0, 39321, 39321, 39321}      /* pluto    153, 153, 153 - gray60 */  
};

/********** STATIC FUNCTIONS **********/
static void zoom (double level);
static char *scale_label_text (void);
static char *planet_dist_label_text (double dist);
static char *planet_speed_label_text (double speed);
static void redraw (void);
/* callbacks */
static gint delete_event (GtkWidget *widget, GdkEventAny *e, gpointer data);
static void destroy (GtkWidget *widget, gpointer data);
static void new_clicked (GtkWidget *widget, gpointer data);
static void zoom_in_clicked (GtkWidget *widget, gpointer data);
static void zoom_out_clicked (GtkWidget *widget, gpointer data);
static void run_toggle_callback (GtkWidget *widget, gpointer data);
static void burn_toggle_callback (GtkWidget *widget, gpointer data);
static void angle_value_changed (GtkObject *adjustment, gpointer data);
static void acc_value_changed (GtkObject *adjustment, gpointer data);
static void time_value_changed (GtkObject *adjustment, gpointer data);
static gint drawing_area_configure (GtkWidget *widget, 
                                    GdkEventConfigure *event, gpointer data);
static gint drawing_area_expose (GtkWidget *widget,
                                 GdkEventExpose *event, gpointer data);
static gboolean drawing_area_button (GtkWidget *widget,
                                     GdkEventButton *event, gpointer data);
static gboolean drawing_area_motion (GtkWidget *widget,
                                     GdkEventMotion *event, gpointer data);
static gint simulation_timeout (gpointer data);

/*********** IMPLEMENTATION ***********/
int 
main (int argc, char *argv[])
{
  GtkWidget *window;
  GtkWidget *main_vbox;
  GtkWidget *hbox;
  GtkWidget *cntl_vbox;
  GtkWidget *sim_frame;
  GtkWidget *sim_vbox;
  GtkWidget *quit_btn;
  GtkWidget *data_frame;
  GtkWidget *data_vbox;
  GtkWidget *con_frame;
  GtkWidget *con_vbox;
  GtkWidget *angle_hbox;
  GtkWidget *angle_label;
  GtkWidget *angle_spin;
  GtkWidget *acc_label;
  GtkWidget *acc_scale;
  GtkWidget *time_frame;
  GtkObject *time_adjustment;
  GtkWidget *time_scale;
  GtkWidget *nearest_frame;
  GtkWidget *nearest_vbox;
  int t;

  /* Widget lay-out */
  /* -------------- */
  /* window */
  /* *- main_vbox */
  /*    |- hbox */
  /*    |  |- cntl_vbox */
  /*    |  |  |- sim_frame */
  /*    |  |  |  *- sim_vbox */
  /*    |  |  |     |- quit_btn */
  /*    |  |  |     |- about_btn (from about.h) */
  /*    |  |  |     |- new_btn */
  /*    |  |  |     |- zoom_in_btn */
  /*    |  |  |     |- zoom_out_btn */
  /*    |  |  |     *- run_toggle */
  /*    |  |  |- data_frame */
  /*    |  |  |  *- data_vbox */
  /*    |  |  |     |- scale_label */
  /*    |  |  |     |- nearest_frame */
  /*    |  |  |     |  *- nearest_vbox */
  /*    |  |  |     |     |- planet_name_label */
  /*    |  |  |     |     |- planet_dist_label */
  /*    |  |  |     |     *- planet_speed_label */
  /*    |  |  *- con_frame */
  /*    |  |     *- con_vbox */
  /*    |  |        |- angle_hbox */
  /*    |  |        |  |- angle_label */
  /*    |  |        |  *- angle_spin [0..360 deg.] */
  /*    |  |        |- acc_label */
  /*    |  |        |- acc_scale [0..3 G] */
  /*    |  |        *- burn_toggle */
  /*    |  *- draw_area */
  /*    *- time_frame */
  /*       *-time_scale */

  /* create user interface */
  gtk_init (&argc, &argv);

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), PROGRAM);
  gtk_window_set_wmclass (GTK_WINDOW(window),
                          "main window",
                          "XPlanets");
  gtk_signal_connect (GTK_OBJECT (window), "destroy",
                      GTK_SIGNAL_FUNC (destroy), NULL);
  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
                      GTK_SIGNAL_FUNC (delete_event), NULL);
  gtk_container_border_width (GTK_CONTAINER (window), 2);

  main_vbox = gtk_vbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (window), main_vbox);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_box_pack_start (GTK_BOX(main_vbox), hbox, TRUE, TRUE, 0);

  cntl_vbox = gtk_vbox_new (FALSE, 2);
  gtk_box_pack_start (GTK_BOX(hbox), cntl_vbox, FALSE, FALSE, 0);

  sim_frame = gtk_frame_new ("simulation");
  gtk_box_pack_start (GTK_BOX(cntl_vbox), sim_frame, TRUE, FALSE, 0);

  sim_vbox = gtk_vbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (sim_frame), sim_vbox);
  gtk_container_border_width (GTK_CONTAINER (sim_vbox), 2);

  quit_btn = gtk_button_new_with_label ("quit");
  gtk_signal_connect (GTK_OBJECT (quit_btn), "clicked",
                      GTK_SIGNAL_FUNC (destroy), NULL);
  gtk_box_pack_start (GTK_BOX(sim_vbox), quit_btn, FALSE, FALSE, 0);

  about_btn = gtk_button_new_with_label ("about");
  gtk_signal_connect (GTK_OBJECT (about_btn), "clicked",
                      GTK_SIGNAL_FUNC (about_clicked), 0);
  gtk_box_pack_start (GTK_BOX(sim_vbox), about_btn, FALSE, FALSE, 0);

  new_btn = gtk_button_new_with_label ("reset");
  gtk_signal_connect (GTK_OBJECT (new_btn), "clicked",
                      GTK_SIGNAL_FUNC (new_clicked), 0);
  gtk_box_pack_start (GTK_BOX(sim_vbox), new_btn, FALSE, FALSE, 0);

  zoom_in_btn = gtk_button_new_with_label ("zoom in");
  gtk_box_pack_start (GTK_BOX(sim_vbox), zoom_in_btn, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (zoom_in_btn), "clicked",
                      GTK_SIGNAL_FUNC (zoom_in_clicked), NULL);

  zoom_out_btn = gtk_button_new_with_label ("zoom out");
  gtk_box_pack_start (GTK_BOX(sim_vbox), zoom_out_btn, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (zoom_out_btn), "clicked",
                      GTK_SIGNAL_FUNC (zoom_out_clicked), NULL);

  run_toggle = gtk_toggle_button_new_with_label ("run");
  gtk_signal_connect (GTK_OBJECT (run_toggle), "toggled",
                      GTK_SIGNAL_FUNC (run_toggle_callback), NULL);
  gtk_box_pack_start (GTK_BOX(sim_vbox), run_toggle, FALSE, FALSE, 0);

  data_frame = gtk_frame_new ("data");
  gtk_box_pack_start (GTK_BOX(cntl_vbox), data_frame, TRUE, FALSE, 0);

  data_vbox = gtk_vbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (data_frame), data_vbox);
  gtk_container_border_width (GTK_CONTAINER (data_vbox), 2);

  scale_label = gtk_label_new (scale_label_text());
  gtk_box_pack_start (GTK_BOX(data_vbox), scale_label, FALSE, FALSE, 0);

  nearest_frame = gtk_frame_new ("nearest planet");
  gtk_box_pack_start (GTK_BOX(data_vbox), nearest_frame, FALSE, FALSE, 0);

  nearest_vbox = gtk_vbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (nearest_frame), nearest_vbox);
  gtk_container_border_width (GTK_CONTAINER (nearest_vbox), 2);

  planet_name_label = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX(nearest_vbox), planet_name_label, 
                      FALSE, FALSE, 0);

  planet_dist_label = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX(nearest_vbox), planet_dist_label, 
                      FALSE, FALSE, 0);

  planet_speed_label = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX(nearest_vbox), planet_speed_label, 
                      FALSE, FALSE, 0);

  con_frame = gtk_frame_new ("controls");
  gtk_box_pack_end (GTK_BOX(cntl_vbox), con_frame, TRUE, FALSE, 0);

  con_vbox = gtk_vbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER (con_frame), con_vbox);
  gtk_container_border_width (GTK_CONTAINER (con_vbox), 2);

  angle_hbox = gtk_hbox_new (FALSE, 2);
  gtk_box_pack_start (GTK_BOX(con_vbox), angle_hbox, FALSE, FALSE, 0);

  angle_label = gtk_label_new ("direction [deg]:");
  gtk_box_pack_start (GTK_BOX(angle_hbox), angle_label, FALSE, FALSE, 0);

  angle_adjustment = gtk_adjustment_new (0,      
                                         0,      
                                         360, 
                                         1,      
                                         5,      
                                         0);     
  gtk_signal_connect (GTK_OBJECT (angle_adjustment), "value_changed",
                      GTK_SIGNAL_FUNC (angle_value_changed), 0);
  angle_spin = gtk_spin_button_new (GTK_ADJUSTMENT (angle_adjustment), 0.5, 0);
  gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (angle_spin), 
                                     GTK_UPDATE_IF_VALID);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (angle_spin), TRUE);
  gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (angle_spin), TRUE);
  gtk_box_pack_start (GTK_BOX(angle_hbox), angle_spin, FALSE, FALSE, 0);

  acc_label = gtk_label_new ("accelleration [G]:");
  gtk_box_pack_start (GTK_BOX(con_vbox), acc_label, FALSE, FALSE, 0);

  acc_adjustment = gtk_adjustment_new (1,      
                                      1,      
                                      3, 
                                      0.1,      
                                      0.5,      
                                      0);
  ship.acc = 1*A;
  gtk_signal_connect (GTK_OBJECT (acc_adjustment), "value_changed",
                      GTK_SIGNAL_FUNC (acc_value_changed), 0);
  acc_scale = gtk_hscale_new (GTK_ADJUSTMENT (acc_adjustment));
  gtk_box_pack_start (GTK_BOX(con_vbox), acc_scale, FALSE, FALSE, 0);

  burn_toggle = gtk_toggle_button_new_with_label ("burn");
  gtk_signal_connect (GTK_OBJECT (burn_toggle), "toggled",
                      GTK_SIGNAL_FUNC (burn_toggle_callback), NULL);
  gtk_box_pack_start (GTK_BOX(con_vbox), burn_toggle, FALSE, FALSE, 0);

  draw_area = gtk_drawing_area_new ();
  gtk_widget_add_events (GTK_WIDGET (draw_area), 
                         GDK_BUTTON_MOTION_MASK
                         |GDK_POINTER_MOTION_HINT_MASK
                         |GDK_BUTTON_PRESS_MASK);
  gtk_signal_connect (GTK_OBJECT (draw_area),"configure_event",
                      (GtkSignalFunc) drawing_area_configure, NULL);
  gtk_signal_connect (GTK_OBJECT (draw_area), "expose_event",
                      (GtkSignalFunc) drawing_area_expose, NULL);
  gtk_signal_connect (GTK_OBJECT (draw_area), "button_press_event",
                      (GtkSignalFunc) drawing_area_button, NULL);
  gtk_signal_connect (GTK_OBJECT (draw_area), "motion_notify_event",
                      (GtkSignalFunc) drawing_area_motion, NULL);

  gtk_drawing_area_size (GTK_DRAWING_AREA (draw_area), DA_SIZE, DA_SIZE);
  gtk_box_pack_start (GTK_BOX (hbox), draw_area, TRUE, TRUE, 0);

  time_frame = gtk_frame_new ("time accelleration factor");
  gtk_box_pack_start (GTK_BOX (main_vbox), time_frame, FALSE, FALSE, 0);

  time_adjustment = gtk_adjustment_new (1,      /* value */
                                        1,      /* lower */
                                        50000, /* upper */
                                        5,      /* step_inc */
                                        10,     /* page_inc */
                                        0);     /* page_size */
  gtk_signal_connect (GTK_OBJECT (time_adjustment), "value_changed",
                      GTK_SIGNAL_FUNC (time_value_changed), 0);
  time_scale = gtk_hscale_new (GTK_ADJUSTMENT (time_adjustment));
  gtk_container_add (GTK_CONTAINER (time_frame), time_scale);
  gtk_widget_show_all (window);

  pmgc = gdk_gc_new (draw_area->window);
  for (t = 0; t < NUM_PLANETS; t++) {
    gdk_color_alloc (gdk_colormap_get_system (), &color[t]);
  }
  gdk_color_alloc (gdk_colormap_get_system (), &ship_color);
  gdk_color_alloc (gdk_colormap_get_system (), &prox_color);
  gdk_color_alloc (gdk_colormap_get_system (), &line_color);
  gdk_color_alloc (gdk_colormap_get_system (), &orbit_color);

  /* initialize the simulation */
  pl_init (draw_area->allocation.width, draw_area->allocation.height);
  zoom (zoomlevel);
  simulation_timeout ((gpointer)FALSE);

  /* gtk main loop */
  gtk_main ();

  /* clean up */
  return (0);
}

void 
zoom (double level)
{
  double size;
  
  size = pow (10, zoomlevel);
  pl_set_view_size (size, size);
}

char *
scale_label_text (void)
{
  static char label_text[25];

  memset (label_text, 0, 25);
  g_snprintf (label_text, 25, "window: 1e%g m", zoomlevel);
  return (char*) label_text;  
}

char *
planet_dist_label_text (double dist)
{
  static char text[25];
  
  memset (text, 0, 25);
  g_snprintf (text, 25, "%g km", dist/1000);
  return (char*) text;  
}

char *
planet_speed_label_text (double speed)
{
  static char text[25];
  
  memset (text, 0, 25);
  g_snprintf (text, 25, "%8.3f km/s", speed/1000);
  return (char*) text;  
}

gint 
delete_event (GtkWidget *widget, GdkEventAny *e, gpointer data)
{
  return FALSE; /* continue with the delete */
}

void 
destroy (GtkWidget *widget, gpointer data)
{
  gtk_main_quit ();
}

void 
new_clicked (GtkWidget *widget, gpointer data)
{
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (run_toggle), FALSE);
  pl_set_ship_std_orbit (3);
  zoomlevel = 12;
  zoom (zoomlevel);
  simulation_timeout ((gpointer)FALSE);
}

void 
zoom_in_clicked (GtkWidget *widget, gpointer data)
{
  if (zoomlevel > MIN_ZOOM) {
    zoomlevel -= 0.5;
    gtk_label_set_text (GTK_LABEL (scale_label), scale_label_text ());
    if (zoomlevel <= MIN_ZOOM) {
      gtk_widget_set_sensitive (zoom_in_btn, FALSE);
    }
    if (!GTK_WIDGET_IS_SENSITIVE(zoom_out_btn)) {
      gtk_widget_set_sensitive (zoom_out_btn, TRUE);
    }
    zoom (zoomlevel);
    simulation_timeout ((gpointer)FALSE);
  }
}

void 
zoom_out_clicked (GtkWidget *widget, gpointer data)
{
  if (zoomlevel < MAX_ZOOM) {
    zoomlevel += 0.5;
    gtk_label_set_text (GTK_LABEL (scale_label), scale_label_text ());
    if (zoomlevel >= MAX_ZOOM) {
      gtk_widget_set_sensitive (zoom_out_btn, FALSE);
    }
    if (!GTK_WIDGET_IS_SENSITIVE(zoom_in_btn)) {
      gtk_widget_set_sensitive (zoom_in_btn, TRUE);
    }
    zoom (zoomlevel);
    simulation_timeout ((gpointer)FALSE);
  }
}

void 
run_toggle_callback (GtkWidget *widget, gpointer data)
{
  if (GTK_TOGGLE_BUTTON (widget)->active) {
    sim_state = SIM_RUN;
    gtk_timeout_add (TMOUT, (GtkFunction) simulation_timeout, (gpointer)TRUE);
  } else {
    sim_state = SIM_STOP;
  }
}

void burn_toggle_callback (GtkWidget *widget, gpointer data)
{
  if (GTK_TOGGLE_BUTTON (widget)->active) {
    ship.engine_on = 1;
  } else {
    ship.engine_on = 0;
  }
}

void 
angle_value_changed (GtkObject *adjustment, gpointer data)
{
  /* ship.dir = rad, adjustment = degrees */
  ship.dir = GTK_ADJUSTMENT(adjustment)->value*M_PI/180;
  if (pixmap)
    redraw();
}

void 
acc_value_changed (GtkObject *adjustment, gpointer data)
{
  /* acc = m/s^2, adjustment = G m/s^2*/
  ship.acc = GTK_ADJUSTMENT(adjustment)->value*A;
  if (pixmap)
    redraw();
}

void 
time_value_changed (GtkObject *adjustment, gpointer data)
{
  time_acc_fact = GTK_ADJUSTMENT(adjustment)->value;
}

gint 
drawing_area_configure (GtkWidget *widget, 
                        GdkEventConfigure *event, gpointer data)
{
  if (pixmap) {
    gdk_pixmap_unref (pixmap);
  }
  pixmap = gdk_pixmap_new (widget->window,
                           widget->allocation.width,
                           widget->allocation.height,
                           -1);
  gdk_draw_rectangle (pixmap,
                      widget->style->black_gc,
                      TRUE,
                      0, 0,
                      widget->allocation.width,
                      widget->allocation.height);
  pl_set_view_screen (widget->allocation.width, 
                      widget->allocation.height);
  if (pmgc)
    simulation_timeout ((gpointer)FALSE);    
  return TRUE;
}

gint 
drawing_area_expose (GtkWidget *widget,
                     GdkEventExpose *event, gpointer data)
{
  gdk_draw_pixmap(widget->window,
                  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                  pixmap,
                  event->area.x, event->area.y,
                  event->area.x, event->area.y,
                  event->area.width, event->area.height);
  return FALSE;
}


gboolean 
drawing_area_button (GtkWidget *widget,
                     GdkEventButton *event, gpointer data)
{
  int i, num = -1;

  /* find the index of the planet that has been clicked on (if any) */
  for (i = 0; i < NUM_PLANETS; i++) {
    if (planet[i].on_screen) {
      int dx, dy;
      dx = (int)event->x - planet[i].screen.x;
      dx *= dx;
      dy = (int)event->y - planet[i].screen.y;
      dy *= dy;
      if (dx+dy < planet[i].r_screen * planet[i].r_screen) {
        num = i;
        break;
      }
    }
  }
  if (num > 0) {
    show_planet_dialog (num);
  } else {
    drawing_area_motion (widget, (GdkEventMotion *)event, (gpointer)1);
  }
  return TRUE; /* event is handled. */
}


gboolean 
drawing_area_motion (GtkWidget *widget,
                     GdkEventMotion *event, gpointer data)
{
  double dir, xd, yd;
  gint x, y;

  x = (gint) event->x;
  y = (gint) event->y;
  if (data == NULL && event->is_hint) {
    gdk_window_get_pointer(event->window, &x, &y, NULL);
  }
  
  xd = x - ship.screen.x;
  yd = -y + ship.screen.y;
  dir = atan2 (yd, xd); /* radians */
  if (dir < 0)
    dir = 2*M_PI + dir;
  ship.dir = dir;
  /* adjustment is in degrees! */
  gtk_adjustment_set_value (GTK_ADJUSTMENT (angle_adjustment), 
                            dir*180.0/M_PI);
  if (pixmap)
    redraw ();
  return TRUE; /* event is handled. */
}

gint 
simulation_timeout (gpointer data)
{
  int i;
  double seconds = time_acc_fact*TMOUT/1000;

/*    struct timeval tv1 = {0,0}, tv2 = {0,0}; */
/*    struct timezone tz; */
/*    double dt; */
/*    gettimeofday (&tv1, &tz); */

  if ((gboolean)data == FALSE) {
    seconds = 0;
  }

  pl_sim_step ((unsigned)seconds);
  if (center_ship) {
    pl_set_view_center (ship.pos.x, ship.pos.y);
  }
  pl_project_to_screen ();
  pl_update_rel ();
  for (i = 0; i < NUM_PLANETS; i++) {
    if (rel_pos[i].len < rel_pos[min_dist_index].len)
      min_dist_index = i;
  }
  gtk_label_set_text (GTK_LABEL (planet_name_label), 
                      planet[min_dist_index].name);
  gtk_label_set_text (GTK_LABEL (planet_dist_label), 
                      planet_dist_label_text (rel_pos[min_dist_index].len));
  gtk_label_set_text (GTK_LABEL (planet_speed_label), 
                      planet_speed_label_text (rel_speed[min_dist_index].len));
  if (rel_pos[min_dist_index].len < 0) {
    /* crash */
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (run_toggle), FALSE);
    show_crash_dialog (min_dist_index);
  }
  redraw ();

/*    gettimeofday (&tv2, &tz); */
/*    if (tv2.tv_usec < tv1.tv_usec) { */
/*      tv2.tv_usec += 1000000; */
/*    } */
/*    dt = tv2.tv_sec - tv1.tv_sec + (tv2.tv_usec - tv1.tv_usec)/1e6; */
/*    printf ("time: %g\n", dt); */

  if (sim_state == SIM_RUN)
    return TRUE;
  return FALSE;
}

void 
redraw (void)
{
  int i;
  ci_t line;
  
  /* clear pixmap, draw planets at new positions */
  gdk_draw_rectangle (pixmap,
                      draw_area->style->black_gc,
                      TRUE,
                      0, 0,
                      draw_area->allocation.width,
                      draw_area->allocation.height);
  for (i = 0; i < NUM_PLANETS; i++) {
/*      if (draw_orbits && i) { */
/*        ci_t *opoints = 0; */
/*        opoints = orbit_get_visible_section (planet[i].orbit); */
/*        gdk_gc_set_foreground (pmgc, &orbit_color); */
/*        if (opoints) { */
/*          int t; */
/*          for (t = 0; t < 36-1; t++) { */
/*            gdk_draw_line (pixmap, pmgc,  */
/*                           opoints[t].x,  */
/*                           opoints[t].y, */
/*                           opoints[t+1].x,  */
/*                           opoints[t+1].y); */
/*          } */
/*        } */
/*      } */
    if (planet[i].on_screen == TRUE) {
      int xs = planet[i].screen.x-planet[i].r_screen;
      int ys = planet[i].screen.y-planet[i].r_screen;
      int sz = planet[i].r_screen*2;
      
      gdk_gc_set_foreground (pmgc, &color[i]);
      gdk_draw_arc (pixmap, pmgc, TRUE,
                    xs, ys,
                    sz, sz, 
                    0, 360*64);
      if (min_dist_index == i) {
        gdk_gc_set_foreground (pmgc, &prox_color);
        gdk_draw_arc (pixmap, pmgc, FALSE, 
                      xs-2, ys-2,
                      sz+4, sz+4, 
                      0, 360*64);
      }
    }
  }
  /* draw relative velocity vectors */
  gdk_gc_set_foreground (pmgc, &ship_color);
  for (i = 0; i < NUM_PLANETS; i++) {
    if (planet[i].on_screen == TRUE) {
      line.x = planet[i].screen.x 
        + (rel_speed[i].len + 50000) * cos (-rel_speed[i].phi)/(V_SCALE*1000);
      line.y = planet[i].screen.y 
        + (rel_speed[i].len + 50000) * sin (-rel_speed[i].phi)/(V_SCALE*1000);
      gdk_draw_line (pixmap, pmgc, 
                     planet[i].screen.x, 
                     planet[i].screen.y,
                     line.x, line.y);
    }
  }
  /* draw the ship */
  if (ship.on_screen == TRUE) {
    line.x = ship.screen.x + V_LEN*cos (ship.dir);
    line.y = ship.screen.y - V_LEN*sin (ship.dir);
    gdk_gc_set_foreground (pmgc, &line_color);
    gdk_draw_line (pixmap, pmgc, 
                   ship.screen.x, 
                   ship.screen.y,
                   line.x, line.y);
    gdk_gc_set_foreground (pmgc, &ship_color);
    gdk_draw_arc (pixmap, pmgc,
                  FALSE, ship.screen.x-3, ship.screen.y-3,
                  6, 6, 0, 360*64);
  }
  /* refresh the screen */
  gdk_draw_pixmap(draw_area->window,
                  draw_area->style->fg_gc[GTK_WIDGET_STATE (draw_area)],
                  pixmap,
                  0, 0,
                  0, 0,
                  draw_area->allocation.width, 
                  draw_area->allocation.height);
}
